import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, notification, Modal, Button } from 'antd'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import asyncComponent from '@/utils/asyncComponent'
import asyncSpinComponent from '@/utils/asyncSpinComponent'
import MKEmitter from '@/utils/events.js'
import zhCN from '@/locales/zh-CN/main.js'
import enUS from '@/locales/en-US/main.js'
import './index.scss'

const MutilForm = asyncSpinComponent(() => import('@/tabviews/zshare/mutilform'))
const NormalButton = asyncComponent(() => import('@/tabviews/zshare/actionList/normalbutton'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class SimpleForm extends Component {
  static propTpyes = {
    data: PropTypes.array,           // 统一查询数据
    config: PropTypes.object,        // 组件配置信息
    mainSearch: PropTypes.any,       // 外层搜索条件
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    BID: '',
    config: null,
    loading: false,
    sync: false,
    data: null,
    group: null,
    BData: '',
    step: 0
  }

  UNSAFE_componentWillMount () {
    const { data } = this.props
    let config = fromJS(this.props.config).toJS()

    let _data = null
    let _sync = false

    let BID = ''
    let BData = ''

    if (config.setting.supModule) {
      BData = window.GLOB.CacheData.get(config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    if (config.wrap.datatype !== 'static') {
      _sync = config.setting.sync === 'true'

      if (_sync && data) {
        _data = data[config.dataName] || {$$empty: true}
        if (Array.isArray(_data)) {
          _data = _data[0] || {$$empty: true}
        }
        _sync = false
      }
    } else {
      _data = {$$empty: true}
    }

    let _group = config.subcards[0]

    if (_group.subButton.enable === 'false' && (!_group.closeButton || _group.closeButton.enable !== 'true')) {
      _group.subButton.style.display = 'none'
      _group.$button = 'no-button'
    }

    this.setState({
      sync: _sync,
      data: _data,
      group: _group,
      BID: BID || '',
      BData: BData || '',
      config: config,
      arr_field: config.columns.map(col => col.field).join(','),
    }, () => {
      if (config.wrap.datatype !== 'static' && config.setting.sync !== 'true' && config.setting.onload === 'true') {
        setTimeout(() => {
          this.loadData()
        }, config.setting.delay || 0)
      }
    })
  }

  componentDidMount () {
    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('mkFormSubmit', this.mkFormSubmit)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('mkFormSubmit', this.mkFormSubmit)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)
  }

  /**
   * @description 图表数据更新，刷新内容
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { sync, config } = this.state

    if (sync && !is(fromJS(this.props.data), fromJS(nextProps.data))) {
      let _data = {$$empty: true}
      if (nextProps.data && nextProps.data[config.dataName]) {
        _data = nextProps.data[config.dataName]
        if (Array.isArray(_data)) {
          _data = _data[0] || {$$empty: true}
        }
      }

      this.setState({sync: false, data: _data})
    } else if (config.setting.useMSearch && nextProps.mainSearch && !is(fromJS(this.props.mainSearch), fromJS(nextProps.mainSearch))) {
      this.setState({}, () => {
        this.loadData()
      })
    }
  }

  reloadData = (menuId, id) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    this.loadData()
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id) => {
    const { config, group, BID } = this.state

    if (group.uuid !== menuId) return

    if (position === 'mainline' && config.setting.supModule && BID) {
      MKEmitter.emit('reloadData', config.setting.supModule, BID)
    } else {
      this.loadData()
    }

    if (id) {
      MKEmitter.emit('resetSelectLine', config.uuid, id, '')
    }

    this.execSuccess(btn, id)
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    
    if (id !== this.state.BID || id !== '') {
      if (config.wrap.datatype === 'static' || (config.setting.supModule && !id)) {
        this.setState({
          data: null,
          BID: id,
          BData: data
        }, () => {
          this.setState({
            data: {$$empty: true}
          })
        })
      } else {
        this.setState({ BID: id, BData: data }, () => {
          this.loadData()
        })
      }
    }
  }

  execSuccess = (btn, id) => {
    if (btn.linkmenu && btn.linkmenu.length > 0) {
      let menu_id = btn.linkmenu[btn.linkmenu.length - 1]
      let menu = window.GLOB.mkThdMenus.filter(m => m.MenuID === menu_id)[0] || ''

      if (!menu) return

      let newtab = {
        ...menu,
        param: {$BID: id || ''}
      }

      MKEmitter.emit('modifyTabs', newtab, true)
    }
  }

  async loadData () {
    const { mainSearch } = this.props
    const { config, arr_field, BID } = this.state

    if (config.wrap.datatype === 'static') {
      this.setState({
        data: {$$empty: true}
      })
      return
    } else if (config.setting.supModule && !BID) {
      this.setState({
        data: null
      }, () => {
        this.setState({data: {$$empty: true}})
      })
      return
    }

    let searches = config.setting.useMSearch && mainSearch ? mainSearch : []

    let requireFields = searches.filter(item => item.required && item.value === '')
    if (requireFields.length > 0) {
      return
    }

    this.setState({
      loading: true
    })

    let _orderBy = config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, arr_field, searches, _orderBy, 1, 1, BID)

    let result = await Api.genericInterface(param)
    if (result.status) {
      let _data = result.data && result.data[0] ? result.data[0] : {$$empty: true}

      this.setState({
        data: null,
        loading: false
      }, () => {
        this.setState({data: _data})
      })
    } else {
      this.setState({
        loading: false,
      })
      
      if (result.ErrCode === 'N') {
        Modal.error({
          title: result.message,
        })
      } else {
        notification.error({
          top: 92,
          message: result.message,
          duration: 10
        })
      }
    }
  }

  mkFormSubmit = (btnId) => {
    const { group } = this.state

    if (group.uuid !== btnId) return

    this.formRef.handleConfirm().then(res => {
      MKEmitter.emit('triggerFormSubmit', {menuId: btnId, form: res})
    })
  }

  closeTab = () => {
    const { group } = this.state

    MKEmitter.emit('closeTabView', group.subButton.$MenuID)
  }

  render() {
    const { config, loading, BID, BData, data, group, dict } = this.state

    return (
      <div className="custom-simple-form-box" id={'anchor' + config.uuid} style={{...config.style}}>
        {loading ?
          <div className="loading-mask">
            <div className="ant-spin-blur"></div>
            <Spin />
          </div> : null
        }
        <NormalHeader config={config} />
        {data ? <MutilForm
          BID={BID}
          BData={BData}
          dict={dict}
          data={data}
          action={group}
          inputSubmit={() => this.mkFormSubmit(group.uuid)}
          wrappedComponentRef={(inst) => this.formRef = inst}
        /> : null}
        {data ? <div className={'mk-form-action ' + (group.$button || '')}>
          <NormalButton
            BID={BID}
            btn={group.subButton}
            setting={config.setting}
            columns={config.columns}
            selectedData={data.$$empty ? [] : [data]}
          />
          {group.closeButton && group.closeButton.enable === 'true' ? <Button type="link" onClick={this.closeTab} style={group.closeButton.style}>{group.closeButton.label}</Button> : null}
        </div> : null}
      </div>
    )
  }
}

export default SimpleForm